#include "YagimotoNC.h"

long SignalBest=-255;//the best signal achieved so far
long SignalBestX=-255;//the best signal achieved after X axis centering
long SignalBestY=-255;//the best signal achieved after Y axis centering

const int SignalGoodEnough=-50;//a good enough signal such that Yagimoto can stop adjusting

const int SignalMin=-60;//the mimimum signal allowed for contnuing to Y centering

const int ServoX=0;//the servo for the X axis
const int ServoY=2;//the servo for the Y axis

const long ServoInitPosX=1024;//The initial position for the X axis servo
const long ServoInitPosY=880;//The initial position for the Y axis servo

long ServoPosX;//the servo's curent X position
long ServoPosY;//the servo's curent Y position

long BestPosX;//the best position found for the X servo
long BestPosY;//the best position found for the Y servo

long StdCorrectDistX=10;//Standard amount to correct for the X servo
long StdCorrectDistY=10;//Standard amount to correct for the Y servo

int CorrectDistXPrev=0;//the previous correcting distance for the X servo
int CorrectDistYPrev=0;//the previous correcting distance for the Y servo

int CorrectDistXPrev2=0;//the correcting distance for the X servo from 2 times ago
int CorrectDistYPrev2=0;//the correcting distance for the Y servo from 2 times ago

const float TicksPerDegree=2.84305555;//the # of servo ticks per degree of movement

const int ErrorBound=10;//the acceptable amount of error for Signal readings

const int Debug=0;//Whether of not to display debug output

#define ModeOmni 0//mode for an omniderectional antenna (only  Y axis)
#define ModeOmniFine 1//mode for an omniderectional antenna with 1/2 the correcting distance (only  Y axis)
#define ModeFind 2//mode to scan the whole area for  networks
#define ModeSetupX 3//mode to setup for X axis centering
#define ModeSetupY 4//mode to setup for Y axis centering
#define ModeCenterX 5//mode to center the X axis
#define ModeCenterFineX 6//mode to center the X axis with 1/2 the correcting distance
#define ModeCenterY 7//mode to center the Y axis
#define ModeCenterFineY 8//mode to center the Y axis with 1/2 the correcting distance
#define ModeDone 9//Yagimoto has finnished

int Mode=ModeSetupX;

int CheckNewBestPos()
{
    if(Signal>SignalBest && Signal!=0)
    {
        BestPosX=get_servo_position(ServoX);//Set new best X position
        BestPosY=get_servo_position(ServoY);//Set new best Y position
        SignalBest=Signal;
        printf("New Best Signal = %d\n", SignalBest);
        return 0;
    }
    else
    {
        return 1;
    }

}

void NewLastCorrect(int NewCorrectX, int NewCorrectY)
{
    if(NewCorrectX!=0)
    {
        CorrectDistXPrev2=CorrectDistXPrev;
        CorrectDistXPrev=NewCorrectX;
    }

    if(NewCorrectY!=0)
    {
        CorrectDistYPrev2=CorrectDistYPrev;
        CorrectDistYPrev=NewCorrectY;
    }
}

void SetUpServos()
{
    set_servo_position(ServoX,ServoInitPosX);
    set_servo_position(ServoY,ServoInitPosY);

    BestPosX=ServoInitPosX;
    BestPosY=ServoInitPosY;
}

void MoveServo(int CorrectDegX, int CorrectDegY)
{
    ServoPosX=get_servo_position(ServoX);//refresh ServoPosX to the curent value
    ServoPosY=get_servo_position(ServoY);//refresh ServoPosY to the curent value

    set_servo_position(ServoX,ServoPosX+((int)((float)(CorrectDegX))*TicksPerDegree));
    set_servo_position(ServoY,ServoPosY+((int)((float)(-CorrectDegY))*TicksPerDegree));//negative CorrectDegY b/c servo moving down = increase

    NewLastCorrect(CorrectDegX,CorrectDegY);
    printf("Calculated Position Offset = %d,%d\n", CorrectDegX,CorrectDegY);
    msleep(100);
}

int ManualMode()
{
    printf("Manual Mode Enabled\n press Arrow Keys to move Antenna\n Press B to exit\n");
    while(1)
    {
        while(a_button()) msleep(5);
        if(up_button())
        {
            while(up_button()) msleep(5);
            MoveServo(0,StdCorrectDistY);
        }

        if(down_button())
        {
            while(down_button()) msleep(5);
            MoveServo(0,-StdCorrectDistY);
        }

        if(right_button())
        {
            while(right_button()) msleep(5);
            MoveServo(StdCorrectDistX,0);
        }
        if(left_button())
        {
            while(left_button()) msleep(5);
            MoveServo(-StdCorrectDistX,0);
        }

        if(b_button())
		{
            while(b_button()) msleep(5);
            return 0;
        }
        msleep(5);
    }
}

int CalcPosOffset()
{
    RefreshData();

    if(Debug)
    {
        printf("Mode=%d\n", Mode);
    }

    if(Mode==ModeSetupX)
    {
        printf("Running Initial X Setup\n");
        MoveServo(StdCorrectDistX,0);
        RefreshData();
        MoveServo((-2*StdCorrectDistX),0);
        RefreshData();
        Mode=ModeCenterX;
    }

    if(Signal>SignalGoodEnough && SignalPrev>SignalGoodEnough && SignalPrev2>SignalGoodEnough)
    {
        printf("Adequate Signal Achieved at %d\n", Signal);
        Mode=ModeDone;
        return 0;
    }

    if(Signal<SignalPrev && SignalPrev<SignalPrev2 && BestPosX!=ServoInitPosX && BestPosY!=ServoInitPosY)//if signal keeps getting weaker
    {
        printf("Signal keeps getting weaker\nReverting to best known position\n");
        ServoPosX=get_servo_position(ServoX);//refresh ServoPosX to the curent value
        ServoPosY=get_servo_position(ServoY);//refresh ServoPosY to the curent value

        set_servo_position(ServoX,BestPosX);//revert position to best known one
        set_servo_position(ServoY,BestPosY);

        NewLastCorrect((BestPosX-ServoPosX),(BestPosY-ServoPosY));
        return 0;
    }

    if(Signal==-255 && SignalPrev==-255 && BestPosX!=ServoInitPosX && BestPosY!=ServoInitPosY)//if we lost the network
    {
        printf("Lost network\nReverting to best known position\n");
        ServoPosX=get_servo_position(ServoX);//refresh ServoPosX to the curent value
        ServoPosY=get_servo_position(ServoY);//refresh ServoPosY to the curent value

        set_servo_position(ServoX,BestPosX);
        set_servo_position(ServoY,BestPosY);

        NewLastCorrect((BestPosX-ServoPosX),(BestPosY-ServoPosY));
        return 0;
    }

    if(Mode==ModeCenterX || Mode==ModeCenterFineX)
    {
        if(Signal>=(SignalBest-ErrorBound) && SignalPrev>=(SignalBest-ErrorBound) && SignalPrev2>=(SignalBest-ErrorBound) && Mode==ModeCenterX)//if best results are achieved with ModeCenterX
        {
            printf("Done with ModeCenterX\n Max Signal Achieved at %d\n", SignalBest);
            Mode=ModeCenterFineX;
            StdCorrectDistX=StdCorrectDistX/2;
        }

        if(Signal>=(SignalBest-ErrorBound/2) && SignalPrev>=(SignalBest-ErrorBound/2) && SignalPrev2>=(SignalBest-ErrorBound/2) && Mode==ModeCenterFineX && SignalBest>SignalMin)//if best results are achieved with ModeCenterFineX
        {
            printf("Done with ModeCenterFineX\n Max Signal Achieved at %d\n", SignalBest);
            SignalBestX=SignalBest;
            Mode=ModeSetupY;
            StdCorrectDistX=StdCorrectDistX*2;
            return 0;
        }

        if(Signal>SignalPrev && SignalPrev>SignalPrev2)//if signal is always getting stronger
        {
            printf("Signal Keeps Getting Stronger\n");
            if(CorrectDistXPrev>0)//if antenna went right last time
            {
                if(CorrectDistXPrev2>0)//if antenna went right then right
                {
                    printf("Antenna moved right then right and Signal kept getting stronger, going right again\n");
                    MoveServo(StdCorrectDistX,0);
                    return 0;
                }

                if(CorrectDistXPrev2<0)//if antenna went left then right
                {
                    printf("Antenna moved left then right and Signal kept getting stronger, going inbetween previous pts\n");
                    MoveServo((int)(-StdCorrectDistX/4),0);
                    return 0;
                }
            }

            if(CorrectDistXPrev<0)//if antenna went left last time
            {
                if(CorrectDistXPrev2>0)//if antenna went right then left
                {
                    printf("Antenna moved right then left and Signal kept getting stronger, going inbetween previous pts\n");
                    MoveServo((int)(StdCorrectDistX/4),0);
                    return 0;
                }

                if(CorrectDistXPrev2<0)//if antenna went left then left
                {
                    printf("Antenna moved left then left and Signal kept getting stronger, going left again\n");
                    MoveServo(-StdCorrectDistX,0);
                    return 0;
                }
            }
        }

        if(Signal<SignalPrev && SignalPrev<SignalPrev2)//if signal is always getting weaker
        {
            printf("Signal Keeps Getting Weaker\n");
            if(CorrectDistXPrev>0)//if antenna went right last time
            {
                if(CorrectDistXPrev2>0)//if antenna went right then right
                {
                    printf("Antenna moved right then right and Signal kept getting weaker, going left to correct\n");
                    MoveServo(-(CorrectDistXPrev+CorrectDistXPrev),0);
                    return 0;
                }

                if(CorrectDistXPrev2<0)//if antenna went left then right
                {
                    printf("Antenna moved left then right and Signal kept getting weaker, going left to correct\n");
                    MoveServo(-CorrectDistXPrev,0);
                    return 0;
                }
            }

            if(CorrectDistXPrev<0)//if antenna went left last time
            {
                if(CorrectDistXPrev2>0)//if antenna went right then left
                {
                    printf("Antenna moved right then left and Signal kept getting weaker, going right to correct\n");
                    MoveServo(-CorrectDistXPrev,0);
                    return 0;
                }

                if(CorrectDistXPrev2<0)//if antenna went left then left
                {
                    printf("Antenna moved left then left and Signal kept getting weaker, going right to correct\n");
                    MoveServo((abs(CorrectDistXPrev)+abs(CorrectDistXPrev)),0);
                    return 0;
                }
            }
        }

        if(Signal>SignalPrev && SignalPrev<SignalPrev2)//if the signal got weaker then stronger
        {
            if(CorrectDistXPrev>0)//if antenna went right last time
            {
                if(CorrectDistXPrev2>0)//if antenna went right then right
                {
                    printf("Antenna moved right then right and got weaker then stronger, going right to correct\n");
                    MoveServo(StdCorrectDistX,0);
                    return 0;
                }

                if(CorrectDistXPrev2<0)//if antenna went left then right
                {
                    printf("Antenna moved left then right and got weaker then stronger, going right to correct\n");
                    MoveServo(StdCorrectDistX,0);
                    return 0;
                }
            }

            if(CorrectDistXPrev<0)//if antenna went left last time
            {
                if(CorrectDistXPrev2>0)//if antenna went right then left
                {
                    printf("Antenna moved right then left and got weaker then better, going left to correct\n");
                    MoveServo(-StdCorrectDistX,0);
                    return 0;
                }

                if(CorrectDistXPrev2<0)//if antenna went left then left
                {
                    printf("Antenna moved left then left and got weaker then better, going left to correct\n");
                    MoveServo(-StdCorrectDistX,0);
                    return 0;
                }
            }
        }
    }

    if(Mode==ModeSetupY)
    {
        printf("Running Initial Y Setup\n");
        MoveServo(0,StdCorrectDistY);
        RefreshData();
        MoveServo(0,(-2*StdCorrectDistY));
        RefreshData();
        Mode=ModeCenterY;
    }

    if(Mode==ModeCenterY || Mode==ModeCenterFineY || Mode==ModeOmni || Mode==ModeOmniFine)
    {
        if(Signal>=(SignalBest-ErrorBound) && SignalPrev>=(SignalBest-ErrorBound) && SignalPrev2>=(SignalBest-ErrorBound) && Mode==ModeCenterY || Mode ==ModeOmni)//if best results are achieved with ModeCenterX
        {
            if(Mode==ModeOmni)
            {
                printf("Done with ModeOmni\n Max Signal Achieved at %d\n", SignalBest);
                Mode=ModeOmniFine;
            }
            else
            {
                printf("Done with ModeCenterY\n Max Signal Achieved at %d\n", SignalBest);
                Mode=ModeCenterFineY;
            }

            StdCorrectDistY=StdCorrectDistY/2;
        }

        if(Signal>=(SignalBest-ErrorBound/2) && SignalPrev>=(SignalBest-ErrorBound/2) && SignalPrev2>=(SignalBest-ErrorBound/2) && Mode==ModeCenterFineX && SignalBest>SignalMin && Signal>(SignalBestX-5) && (Mode==ModeCenterFineY || Mode==ModeOmniFine))//if best results are achieved with ModeCenterFineX
        {
            if(Mode==ModeOmni)
            {
                printf("Done with ModeOmniFine\n Max Signal Achieved at %d\n", SignalBest);
            }
            else
            {
                printf("Done with ModeCenterFineY\n Max Signal Achieved at %d\n", SignalBest);
                Mode=ModeDone;
                StdCorrectDistY=StdCorrectDistY*2;
                return 0;
            }
        }

        if(Signal>SignalPrev && SignalPrev>SignalPrev2)//if signal is always getting stronger
        {
            printf("Signal Keeps Getting Stronger\n");
            if(CorrectDistYPrev>0)//if antenna went right last time
            {
                if(CorrectDistYPrev2>0)//if antenna went up then up
                {
                    printf("Antenna moved up then up and Signal kept getting stronger, going up again\n");
                    MoveServo(0,StdCorrectDistY);
                    return 0;
                }

                if(CorrectDistYPrev2<0)//if antenna went down then up
                {
                    printf("Antenna moved down then up and Signal kept getting stronger, going inbetween previous pts\n");
                    MoveServo(0,(int)(-StdCorrectDistY/4));
                    return 0;
                }
            }

            if(CorrectDistYPrev<0)//if antenna went down last time
            {
                if(CorrectDistYPrev2>0)//if antenna went up then down
                {
                    printf("Antenna moved up then down and Signal kept getting stronger, going inbetween previous pts\n");
                    MoveServo(0,(int)(StdCorrectDistY/4));
                    return 0;
                }

                if(CorrectDistYPrev2<0)//if antenna went down then down
                {
                    printf("Antenna moved down then down and Signal kept getting stronger, going down again\n");
                    MoveServo(0,-StdCorrectDistY);
                    return 0;
                }
            }
        }

        if(Signal<SignalPrev && SignalPrev<SignalPrev2)//if signal is always getting weaker
        {
            printf("Signal Keeps Getting Weaker\n");
            if(CorrectDistYPrev>0)//if antenna went up last time
            {
                if(CorrectDistYPrev2>0)//if antenna went up then up
                {
                    printf("Antenna moved up then up and Signal kept getting weaker, going down to correct\n");
                    MoveServo(0,-(CorrectDistYPrev+CorrectDistYPrev));
                    return 0;
                }

                if(CorrectDistYPrev2<0)//if antenna went down then up
                {
                    printf("Antenna moved down then up and Signal kept getting weaker, going down to correct\n");
                    MoveServo(0,-CorrectDistYPrev);
                    return 0;
                }
            }

            if(CorrectDistYPrev<0)//if antenna went down last time
            {
                if(CorrectDistYPrev2>0)//if antenna went up then down
                {
                    printf("Antenna moved up then down and Signal kept getting weaker, going up to correct\n");
                    MoveServo(0,-CorrectDistYPrev);
                    return 0;
                }

                if(CorrectDistYPrev2<0)//if antenna went down then down                {
                    printf("Antenna moved down then down and Signal kept getting weaker, going up to correct\n");
                    MoveServo(0,(abs(CorrectDistYPrev)+abs(CorrectDistYPrev)));
                    return 0;
                }
            }


        if(Signal>SignalPrev && SignalPrev<SignalPrev2)//if the signal got weaker then better
        {
            if(CorrectDistYPrev>0)//if antenna went up last time
            {
                if(CorrectDistYPrev2>0)//if antenna went up then up
                {
                    printf("Antenna moved up then up and got weaker then better, going up to correct\n");
                    MoveServo(0,StdCorrectDistY);
                    return 0;
                }

                if(CorrectDistXPrev2<0)//if antenna went down then up
                {
                    printf("Antenna moved down then up and got weaker then better, going up to correct\n");
                    MoveServo(0,StdCorrectDistY);
                    return 0;
                }
            }

            if(CorrectDistYPrev<0)//if antenna went down last time
            {
                if(CorrectDistYPrev2>0)//if antenna went up then down
                {
                    printf("Antenna moved up then down and got weaker then better, going down to correct\n");
                    MoveServo(0,-StdCorrectDistY);
                    return 0;
                }

                if(CorrectDistYPrev2<0)//if antenna went down then down
                {
                    printf("Antenna moved down then down and got weaker then better, going down to correct\n");
                    MoveServo(0,-StdCorrectDistY);
                    return 0;
                }
            }
        }
    }

    if(Signal<SignalPrev && SignalPrev>SignalPrev2)//if the signal got better then worse
    {
        printf("Signal got better then weaker, going one step farther to confirm signal loss\n");
        MoveServo(CorrectDistXPrev,CorrectDistYPrev);
        return 0;
    }
}

int main()
{
    SetUpServos();
	SetUpNetcat();
	while(Mode != ModeDone)
	{
	    if(a_button())
		{
		    ManualMode();
		}

		CalcPosOffset();
	}
	printf("Mode=Done");
	return 0;
}
